From: Baoquan He <bhe@redhat.com>
To: linux-kernel@vger.kernel.org
Cc: x86@kernel.org, hpa@zytor.com, tglx@linutronix.de,
mingo@kernel.org, keescook@chromium.org, thgarnie@google.com,
caoj.fnst@cn.fujitsu.com, izumi.taku@jp.fujitsu.com,
Baoquan He <bhe@redhat.com>
Subject: [PATCH v2 2/2] x86/boot/KASLR: Restrict kernel to be randomized in mirror regions
Date: Tue, 4 Jul 2017 16:04:02 +0800 [thread overview]
Message-ID: <1499155442-17467-3-git-send-email-bhe@redhat.com> (raw)
In-Reply-To: <1499155442-17467-1-git-send-email-bhe@redhat.com>
Kernel text may be located in non-mirror regions (movable zone) when both
address range mirroring feature and KASLR are enabled.
The address range mirroring feature arranges such mirror region into
normal zone and other region into movable zone in order to locate
kernel code and data in mirror region. The physical memory region
whose descriptors in EFI memory map has EFI_MEMORY_MORE_RELIABLE
attribute (bit: 16) are mirrored.
If efi is detected, iterate efi memory map and pick the mirror region to
process for adding candidate of randomization slot. If efi is disabled
or no mirror region found, still process e820 memory map.
Signed-off-by: Baoquan He <bhe@redhat.com>
---
arch/x86/boot/compressed/kaslr.c | 52 +++++++++++++++++++++++++++++++++++++++-
1 file changed, 51 insertions(+), 1 deletion(-)
diff --git a/arch/x86/boot/compressed/kaslr.c b/arch/x86/boot/compressed/kaslr.c
index 85c360eec4a6..5f10e0b10ef4 100644
--- a/arch/x86/boot/compressed/kaslr.c
+++ b/arch/x86/boot/compressed/kaslr.c
@@ -37,7 +37,9 @@
#include <linux/uts.h>
#include <linux/utsname.h>
#include <linux/ctype.h>
+#include <linux/efi.h>
#include <generated/utsrelease.h>
+#include <asm/efi.h>
/* Macros used by the included decompressor code below. */
#define STATIC
@@ -558,6 +560,50 @@ static void process_mem_region(struct mem_vector *entry,
}
}
+/* Marks if efi mirror regions have been found and handled. */
+static bool efi_mirror_found;
+
+static void process_efi_entry(unsigned long minimum, unsigned long image_size)
+{
+ struct efi_info *e = &boot_params->efi_info;
+ struct mem_vector region;
+ efi_memory_desc_t *md;
+ unsigned long pmap;
+ char *signature;
+ u32 nr_desc;
+ int i;
+
+
+#ifdef CONFIG_EFI
+ signature = (char *)&boot_params->efi_info.efi_loader_signature;
+#endif
+ if (strncmp(signature, EFI32_LOADER_SIGNATURE, 4) &&
+ strncmp(signature, EFI64_LOADER_SIGNATURE, 4))
+ return;
+
+#ifdef CONFIG_X86_32
+ /* Can't handle data above 4GB at this time */
+ if (e->efi_memmap_hi) {
+ warn("Memory map is above 4GB, EFI should be disabled.\n");
+ return;
+ }
+ pmap = e->efi_memmap;
+#else
+ pmap = (e->efi_memmap | ((__u64)e->efi_memmap_hi << 32));
+#endif
+
+ nr_desc = e->efi_memmap_size / e->efi_memdesc_size;
+ for (i = 0; i < nr_desc; i++) {
+ md = (efi_memory_desc_t *)(pmap + (i * e->efi_memdesc_size));
+ if (md->attribute & EFI_MEMORY_MORE_RELIABLE) {
+ region.start = md->phys_addr;
+ region.size = md->num_pages << EFI_PAGE_SHIFT;
+ process_mem_region(®ion, minimum, image_size);
+ efi_mirror_found = true;
+ }
+ }
+}
+
static void process_e820_entry(unsigned long minimum, unsigned long image_size)
{
int i;
@@ -592,6 +638,10 @@ static unsigned long find_random_phys_addr(unsigned long minimum,
/* Make sure minimum is aligned. */
minimum = ALIGN(minimum, CONFIG_PHYSICAL_ALIGN);
+ process_efi_entry(minimum, image_size);
+ if (efi_mirror_found)
+ return slots_fetch_random();
+
process_e820_entry(minimum, image_size);
return slots_fetch_random();
}
@@ -651,7 +701,7 @@ void choose_random_location(unsigned long input,
*/
min_addr = min(*output, 512UL << 20);
/* Walk e820 and find a random address. */
random_addr = find_random_phys_addr(min_addr, output_size);
if (!random_addr) {
warn("Physical KASLR disabled: no suitable memory region!");
@@ -651,7 +701,7 @@ void choose_random_location(unsigned long input,
*/
min_addr = min(*output, 512UL << 20);
- /* Walk e820 and find a random address. */
+ /* Walk available memory entries to find a random address. */
random_addr = find_random_phys_addr(min_addr, output_size);
if (!random_addr) {
--
2.5.5
next prev parent reply other threads:[~2017-07-04 8:04 UTC|newest]
Thread overview: 11+ messages / expand[flat|nested] mbox.gz Atom feed top
2017-07-04 8:04 [PATCH v2 0/2] x86/boot/KASLR: Restrict kernel to be randomized in mirror regions Baoquan He
2017-07-04 8:04 ` [PATCH v2 1/2] x86/boot/KASLR: Adapt process_e820_entry for any type of memory entry Baoquan He
2017-07-05 22:06 ` Kees Cook
2017-07-06 1:21 ` Baoquan He
2017-07-04 8:04 ` Baoquan He [this message]
2017-07-04 14:00 ` [PATCH v2 2/2] x86/boot/KASLR: Restrict kernel to be randomized in mirror regions Thomas Gleixner
2017-07-04 14:30 ` Baoquan He
2017-07-04 14:46 ` Thomas Gleixner
2017-07-04 15:36 ` Matt Fleming
2017-07-04 15:46 ` Thomas Gleixner
2017-07-04 23:16 ` Baoquan He
Reply instructions:
You may reply publicly to this message via plain-text email
using any one of the following methods:
* Save the following mbox file, import it into your mail client,
and reply-to-all from there: mbox
Avoid top-posting and favor interleaved quoting:
https://en.wikipedia.org/wiki/Posting_style#Interleaved_style
* Reply using the --to, --cc, and --in-reply-to
switches of git-send-email(1):
git send-email \
--in-reply-to=1499155442-17467-3-git-send-email-bhe@redhat.com \
--to=bhe@redhat.com \
--cc=caoj.fnst@cn.fujitsu.com \
--cc=hpa@zytor.com \
--cc=izumi.taku@jp.fujitsu.com \
--cc=keescook@chromium.org \
--cc=linux-kernel@vger.kernel.org \
--cc=mingo@kernel.org \
--cc=tglx@linutronix.de \
--cc=thgarnie@google.com \
--cc=x86@kernel.org \
/path/to/YOUR_REPLY
https://kernel.org/pub/software/scm/git/docs/git-send-email.html
* If your mail client supports setting the In-Reply-To header
via mailto: links, try the mailto: link
Be sure your reply has a Subject: header at the top and a blank line
before the message body.
This is an external index of several public inboxes,
see mirroring instructions on how to clone and mirror
all data and code used by this external index.